cordova.define("@outsystems/cordova-outsystems-appshield.shield-manager", function(require, exports, module) {
// Manages collections of listeners and events. Registers a channel that
// receives events and propagates those events via registered listeners.
// Queues events that the channel emits until at least one listener has
// been registered. When the first one is set, the queued events are
// propagated. Events that are emitted when at least one listener is
// registered are immediately propagated.
module.exports = function (bridge, logging) {

    // Managed events and listeners
    var events = [];
    var listeners = [];

    // Registers a listener
    // All queued events are emitted
    // New events will be immediately emitted
    this.registerListener = function (listener) {
        if (listener) {
            listeners.push(listener);

            propagateQueuedEvents();
        }
    }

    // Unregisters a listener
    this.unregisterListener = function (listener) {
        var index = listeners.indexOf(listener);
        if (index >= 0) {
            listeners.splice(index, 1);
        }
    }

    // Method that handles an event emitted by the channel
    // If no listeners are registered, the event is queued
    // If at least one is registered, the event is emitted via all listeners
    var handleReceivedEvent = function (event) {
        if (listeners.length <= 0) {
            events.push(event);

        } else {
            propagateEvent(event);
        }

        // Logs the event to the Service Center if it's considered important
        logging.logEventIfImportant(event);
    }

    // Emits an event via all listeners
    var propagateEvent = function (event) {
        listeners.forEach(function (listener) {
            setTimeout(function () {
                // TODO: Make this asynchronous if required
                listener(event);
            }, 0);
        });
    }

    // Emits all queued events via all listeners
    var propagateQueuedEvents = function () {
        events.forEach(function (event) {
            propagateEvent(event);
        });

        events.splice(0, events.length);
    }

    // Creates the channel that receives events from the native layer
    bridge.createChannel(handleReceivedEvent);
}

});
