﻿;(function() {
    'use strict';
    
    function setupSessionTimeoutManager(){
        // Create local variable structure
        var SessionTimeoutManager = {};
        
        // Initialise local variables
        SessionTimeoutManager.triggerTimeoutFunction = undefined;
        SessionTimeoutManager.timeoutPeriod = undefined;
        SessionTimeoutManager.timeoutID = undefined;
        SessionTimeoutManager.nextTimeoutTimestamp = undefined;
        SessionTimeoutManager.wasReset = false;
        SessionTimeoutManager.isRestartAfterTimeout = false;
        
        // Triggers session timeout
        SessionTimeoutManager.goInactive = function() {
            SessionTimeoutManager.triggerTimeoutFunction();
            
            window.clearTimeout(SessionTimeoutManager.timeoutID);
            SessionTimeoutManager.nextTimeoutTimestamp = undefined;
            SessionTimeoutManager.setSessionInfo();
        };
        
        // Triggers session timeout and restarts timeout
        SessionTimeoutManager.restart = function() {
            SessionTimeoutManager.triggerTimeoutFunction();
            
            if(SessionTimeoutManager.nextTimeoutTimestamp !== undefined){
                SessionTimeoutManager.wasReset = false;
                window.clearTimeout(SessionTimeoutManager.timeoutID);
            }
            
            SessionTimeoutManager.nextTimeoutTimestamp = undefined;
            
            SessionTimeoutManager.startTimer();
        };
        
        // Starts timeout timer
        SessionTimeoutManager.startTimer = function() {
            if(SessionTimeoutManager.isRestartAfterTimeout)
            {
                SessionTimeoutManager.timeoutID = window.setTimeout(SessionTimeoutManager.restart, SessionTimeoutManager.timeoutPeriod);
            }
            else 
            {
                SessionTimeoutManager.timeoutID = window.setTimeout(SessionTimeoutManager.goInactive, SessionTimeoutManager.timeoutPeriod);
            }
            SessionTimeoutManager.nextTimeoutTimestamp = new Date((new Date()).getTime() + SessionTimeoutManager.timeoutPeriod);
            SessionTimeoutManager.setSessionInfo();
        };
        
        // Resets the timeout timer and variables
        SessionTimeoutManager.resetTimer = function() {
            SessionTimeoutManager.resetFromSession();
            var currDate = new Date();
            var lastExecutionDate = new Date(SessionTimeoutManager.nextTimeoutTimestamp);
            
            if(SessionTimeoutManager.nextTimeoutTimestamp !== undefined && currDate > lastExecutionDate){
               SessionTimeoutManager.goInactive();
            }
            else{
                window.clearTimeout(SessionTimeoutManager.timeoutID);
                SessionTimeoutManager.nextTimeoutTimestamp = undefined;
                
                SessionTimeoutManager.startTimer();
            }
        };
        
        // Stores timeout details in session storage to be reloaded when a OnResume event is trigerred
        SessionTimeoutManager.setSessionInfo = function() {
            var jsonObj = JSON.stringify(SessionTimeoutManager);
            
            sessionStorage.setItem('SessionTimeoutManagerJsonObj', jsonObj);
        };
        
        // Reset timeout details from session storage object
        SessionTimeoutManager.resetFromSession = function() {
            var jsonObj = JSON.parse(sessionStorage.getItem('SessionTimeoutManagerJsonObj'));
            
            if(jsonObj)
            {
                SessionTimeoutManager.timeoutPeriod = jsonObj.timeoutPeriod;
                SessionTimeoutManager.timeoutID = jsonObj.timeoutID;
                SessionTimeoutManager.nextTimeoutTimestamp = jsonObj.nextTimeoutTimestamp;
                SessionTimeoutManager.wasReset = jsonObj.wasReset;
            }
        };

        // Destroy all client events and timer
        SessionTimeoutManager.destroy = function() {
            if(SessionTimeoutManager.wasReset){
               SessionTimeoutManager.wasReset = false;
            }
            else{
                document.body.removeEventListener("mousemove", SessionTimeoutManager.resetTimer, false);
                document.body.removeEventListener("mousedown", SessionTimeoutManager.resetTimer, false);
                document.body.removeEventListener("keypress", SessionTimeoutManager.resetTimer, false);
                document.body.removeEventListener("DOMMouseScroll", SessionTimeoutManager.resetTimer, false);
                document.body.removeEventListener("mousewheel", SessionTimeoutManager.resetTimer, false);
                document.body.removeEventListener("touchmove", SessionTimeoutManager.resetTimer, false);
                document.body.removeEventListener("MSPointerMove", SessionTimeoutManager.resetTimer, false);
                
                
                window.clearTimeout(SessionTimeoutManager.timeoutID);
                
                SessionTimeoutManager.triggerTimeoutFunction = undefined;
                SessionTimeoutManager.timeoutPeriod = undefined;
                SessionTimeoutManager.nextTimeoutTimestamp = undefined;
                
                sessionStorage.removeItem('SessionTimeoutManagerJsonObj');
            }
        };
        
        // Set up session timeout manager.
        SessionTimeoutManager.setup = function (timeoutPeriod, triggerTimeoutFunction, restartAfterTimeout) {
            document.body.addEventListener("mousemove", SessionTimeoutManager.resetTimer, false);
            document.body.addEventListener("mousedown", SessionTimeoutManager.resetTimer, false);
            document.body.addEventListener("keypress", SessionTimeoutManager.resetTimer, false);
            document.body.addEventListener("DOMMouseScroll", SessionTimeoutManager.resetTimer, false);
            document.body.addEventListener("mousewheel", SessionTimeoutManager.resetTimer, false);
            document.body.addEventListener("touchmove", SessionTimeoutManager.resetTimer, false);
            document.body.addEventListener("MSPointerMove", SessionTimeoutManager.resetTimer, false);
            
            window.addEventListener("focus", SessionTimeoutManager);
            
            SessionTimeoutManager.triggerTimeoutFunction = triggerTimeoutFunction;
            
            if(SessionTimeoutManager.nextTimeoutTimestamp !== undefined){
                SessionTimeoutManager.wasReset = true;
                window.clearTimeout(SessionTimeoutManager.timeoutID);
            }
            
            SessionTimeoutManager.timeoutPeriod = timeoutPeriod;
            SessionTimeoutManager.nextTimeoutTimestamp = undefined;
            SessionTimeoutManager.isRestartAfterTimeout = restartAfterTimeout;
            
            SessionTimeoutManager.startTimer();
        };
        
        return SessionTimeoutManager;
    }
    
    if(typeof(SessionTimeoutManager) === 'undefined'){
        window.SessionTimeoutManager = setupSessionTimeoutManager();
    }
}.call(this));