﻿class SendMoney {
    constructor(props) {
        this.version = `[${this.constructor.name}] Version: v1.0`;
        this.classes = {
            el: 'send-money-inputs'
        };
        this.args = {
            el: props.el,
            extendedClass: props.extendedClass,
        };
        this.listeners = {
            click: this.#click.bind(this),
        };
        this.dom = {
            el: this.args.el,
        };
        this.holdExtendedClass = props.extendedClass;
    }
    #error(error) {
        switch (error) {
            case 'extended-class':
                throw Error(`[${this.constructor.name}] The element does not exist, when trying to update EntendedClass '${newCssClass}'.`);
            case 'missing-dependencies':
                throw Error(`[${this.constructor.name}] There is a problem with the dependent libraries.`);
        }
    };
    #setExtendedClass(element, currentCssClasses, newCssClass) {
        if (element) {
            const currentClassesList = currentCssClasses.split(' ');
            const newClassesList = newCssClass.split(' ');
            let classesToRemove = [];
            let classesToAdd = [];
            if (currentCssClasses !== '') {
                classesToRemove = currentClassesList.filter((currClass) => newClassesList.indexOf(currClass) === -1);
            }
            if (newCssClass !== '') {
                classesToAdd = newClassesList.filter((newClass) => currentClassesList.indexOf(newClass) === -1);
            }
            if (classesToRemove.length > 0) {
                classesToRemove.forEach((classToRemove) => {
                    element.classList.remove(classToRemove);
                });
            }
            if (classesToAdd.length > 0) {
                classesToAdd.forEach((classToAdd) => {
                    element.classList.add(classToAdd);
                });
            }
        }
        else {
            this.#error('extended-class');
        }
    };
    #click() {};
    #addEventListeners() {
        const senderEl = document.querySelector('.send-money-inputs__sender');
        const receiverEl = document.querySelector('.send-money-inputs__receiver');
        const parentEl =  document.querySelector('.send-money-inputs');
        const senderInput = senderEl.querySelector('input');
        const receiverInput = receiverEl.querySelector('input');
        var maskedSenderInput = null;
        var maskedReceiverInput = null;

        const senderInputMask = senderEl.querySelector('.input-mask-regex');
        if(senderInputMask !== null) {
            maskedSenderInput = senderInputMask.querySelector('.input-mask-regex--hidden input');
        }

        const receiverInputMask = receiverEl.querySelector('.input-mask-regex');
        if(receiverInputMask !== null) {
            maskedReceiverInput = receiverInputMask.querySelector('.input-mask-regex--hidden input');
        }
        
        senderEl.addEventListener('click',function(){
            setTimeout(function(){senderInput.focus();},10);
            senderEl.classList.add('active');
            receiverEl.classList.remove('active');
            parentEl.classList.remove('receiver-active');
            parentEl.classList.remove('init-active');
            parentEl.classList.add('sender-active');
        });

        if(senderInputMask !== null) {
            if(maskedSenderInput.disabled) {
                parentEl.classList.add('send-money-inputs__sender-disabled');
            }
        } else {
            if(senderInput.disabled) {
                parentEl.classList.add('send-money-inputs__sender-disabled');
            }
        }

        const callbackSender = (mutationList) => {
            for (const mutation of mutationList) {
                if(mutation.target.disabled) {
                    parentEl.classList.add('send-money-inputs__sender-disabled');
                    parentEl.classList.remove('sender-active');
                    senderEl.classList.remove('active');
                } else {
                    parentEl.classList.remove('send-money-inputs__sender-disabled');
                }
            }
        };

        const observerSender = new MutationObserver(callbackSender);
        observerSender.observe(senderInputMask !== null ? maskedSenderInput : senderInput, { attributes: true });
        
        receiverEl.addEventListener('click',function(){
            setTimeout(function(){receiverInput.focus();},10);
            receiverEl.classList.add('active');
            senderEl.classList.remove('active');
            parentEl.classList.remove('sender-active');
            parentEl.classList.remove('init-active');
            parentEl.classList.add('receiver-active');
        });

        if(receiverInputMask !== null) {
            if(maskedReceiverInput.disabled) {
                parentEl.classList.add('send-money-inputs__receiver-disabled');
            }
        } else {
            if(receiverInput.disabled) {
                parentEl.classList.add('send-money-inputs__receiver-disabled');
            }
        }

        if(receiverInput.disabled) {
            parentEl.classList.add('send-money-inputs__receiver-disabled');
        }

        const callbackReceiver = (mutationList) => {
            for (const mutation of mutationList) {
                if(mutation.target.disabled) {
                    parentEl.classList.add('send-money-inputs__receiver-disabled');
                    parentEl.classList.remove('receiver-active');
                    receiverEl.classList.remove('active');
                } else {
                    parentEl.classList.remove('send-money-inputs__receiver-disabled');
                }
            }
        };

        const observerReceiver = new MutationObserver(callbackReceiver);
        observerReceiver.observe(receiverInputMask !== null ? maskedReceiverInput : receiverInput, { attributes: true });
    };
    
    init() {
        //debugger;
        this.#addEventListeners(); 
        this.#setExtendedClass(this.dom.el, '', this.args.extendedClass);
    };
    #storeNewConfig(newConfig) {
        this.args = {...this.args, ...newConfig};
    };
    onParametersChanged(props) {
        for (const key in props) {
            if (props.hasOwnProperty(key)) {
                if (props[key] !== this.args[key]) {
                    switch (key) {
                        case 'extendedClass':
                            this.#setExtendedClass(this.dom.el, this.holdExtendedClass, props.extendedClass);
                            this.holdExtendedClass = props.extendedClass;
                            break;
                    }
                }
            }
        }
        this.#storeNewConfig(props);
    };
    #removeEventListeners() {};
    destroy() {
        this.#removeEventListeners();
    };
};